package net.javase.hksup.starter.service;

import com.sun.jna.Native;
import lombok.extern.slf4j.Slf4j;
import net.javase.hksup.starter.config.HkSupSdkProperties;
import net.javase.hksup.starter.sdk.HCISUPAlarm;
import net.javase.hksup.starter.sdk.HCISUPCMS;
import net.javase.hksup.starter.utils.SystemUtil;
import org.springframework.context.ApplicationContext;

/**
 * AlarmServer
 *
 * @author Frank
 */
@Slf4j
public class AlarmServer {

    private final HkSupSdkProperties hkSupSdkProperties;

    private final ApplicationContext applicationContext;

    public static HCISUPAlarm hcEHomeAlarm = null;
    private int alarmHandle = -1; //Alarm监听句柄
    private HCISUPAlarm.EHomeMsgCallBack cbEHomeMsgCallBack;
    private HCISUPAlarm.NET_EHOME_ALARM_LISTEN_PARAM netEhomeAlarmListenParam = new HCISUPAlarm.NET_EHOME_ALARM_LISTEN_PARAM();

    public AlarmServer(ApplicationContext applicationContext, HkSupSdkProperties hkSupSdkProperties) {
        this.hkSupSdkProperties = hkSupSdkProperties;
        this.applicationContext = applicationContext;
        createSDKInstance();
    }

    /**
     * 初始化CMS服务
     */
    void startServer() {
        try {
            alarmInit();
            startAlarmListen();
        } catch (Exception ex) {
            log.error("启动Alarm服务失败: " + ex.getMessage(), ex);
        }
    }

    /**
     * 停止CMS服务,回收资源
     */
    void stopServer() {
        try {
            if (alarmHandle >= 0) {
                log.info("开始停止Alarm报警服务");
                hcEHomeAlarm.NET_EALARM_StopListen(alarmHandle);
                hcEHomeAlarm.NET_EALARM_Fini();
                log.info("停止Alarm报警服务成功!");
            }
        } catch (Exception ex) {
            log.error("停止Alarm报警服务失败: " + ex.getMessage(), ex);
        }
    }


    /**
     * 根据不同操作系统选择不同的库文件和库路径
     */
    private boolean createSDKInstance() {
        if (hcEHomeAlarm == null) {
            synchronized (HCISUPAlarm.class) {
                String strDllPath = "";
                try {
                    SystemUtil.jnaDebugLoad(hkSupSdkProperties.getJnaDebug());
                    if (SystemUtil.isWindows())
                        //win系统加载库路径(路径不要带中文)
                        strDllPath = hkSupSdkProperties.libPath() + "HCISUPAlarm.dll";
                    else if (SystemUtil.isLinux())
                        //Linux系统加载库路径(路径不要带中文)
                        strDllPath = hkSupSdkProperties.libPath() + "libHCISUPAlarm.so";
                    hcEHomeAlarm = (HCISUPAlarm) Native.loadLibrary(strDllPath, HCISUPAlarm.class);
                } catch (Exception ex) {
                    log.info("loadLibrary: " + strDllPath + " Error: " + ex.getMessage());
                    return false;
                }
            }
        }
        return true;
    }

    private void alarmInit() {
        if (hcEHomeAlarm == null) {
            if (!createSDKInstance()) {
                throw new RuntimeException("Load CMS SDK fail");
            }
        }
        if (SystemUtil.isWindows()) {
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCrypto = hkSupSdkProperties.libPath() + "libeay32.dll"; //Linux版本是libcrypto.so库文件的路径
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, strPathCrypto.length());
            ptrByteArrayCrypto.write();
            hcEHomeAlarm.NET_EALARM_SetSDKInitCfg(0, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathSsl = hkSupSdkProperties.libPath() + "ssleay32.dll";    //Linux版本是libssl.so库文件的路径
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, strPathSsl.length());
            ptrByteArraySsl.write();
            hcEHomeAlarm.NET_EALARM_SetSDKInitCfg(1, ptrByteArraySsl.getPointer());

            //报警服务初始化
            boolean bRet = hcEHomeAlarm.NET_EALARM_Init();
            if (!bRet) {
                log.warn("NET_EALARM_Init failed!");
            }
            //设置HCAapSDKCom组件库文件夹所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCom = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCom = hkSupSdkProperties.libPath() + "HCAapSDKCom";        //只支持绝对路径，建议使用英文路径
            System.arraycopy(strPathCom.getBytes(), 0, ptrByteArrayCom.byValue, 0, strPathCom.length());
            ptrByteArrayCom.write();
            hcEHomeAlarm.NET_EALARM_SetSDKLocalCfg(5, ptrByteArrayCom.getPointer());
        } else if (SystemUtil.isLinux()) {
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCrypto = hkSupSdkProperties.libPath() + "libcrypto.so"; //Linux版本是libcrypto.so库文件的路径
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, strPathCrypto.length());
            ptrByteArrayCrypto.write();
            hcEHomeAlarm.NET_EALARM_SetSDKInitCfg(0, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathSsl = hkSupSdkProperties.libPath() + "libssl.so";    //Linux版本是libssl.so库文件的路径
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, strPathSsl.length());
            ptrByteArraySsl.write();
            hcEHomeAlarm.NET_EALARM_SetSDKInitCfg(1, ptrByteArraySsl.getPointer());
            //报警服务初始化
            boolean bRet = hcEHomeAlarm.NET_EALARM_Init();
            if (!bRet) {
                log.warn("NET_EALARM_Init failed!");
            }
            //设置HCAapSDKCom组件库文件夹所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCom = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCom = hkSupSdkProperties.libPath() + "HCAapSDKCom/";        //只支持绝对路径，建议使用英文路径
            System.arraycopy(strPathCom.getBytes(), 0, ptrByteArrayCom.byValue, 0, strPathCom.length());
            ptrByteArrayCom.write();
            hcEHomeAlarm.NET_EALARM_SetSDKLocalCfg(5, ptrByteArrayCom.getPointer());
        }

        //启用SDK写日志
        boolean logToFile = hcEHomeAlarm.NET_EALARM_SetLogToFile(3, hkSupSdkProperties.logPath(), false);
    }

    /**
     * 开启报警服务监听
     */
    private void startAlarmListen() {
        if (cbEHomeMsgCallBack == null) {
            cbEHomeMsgCallBack = applicationContext.getBeansOfType(HCISUPAlarm.EHomeMsgCallBack.class).values().stream().findFirst().orElse(null);
            if (cbEHomeMsgCallBack == null) {
                throw new RuntimeException("not found EHomeMsgCallBack implement!");
            }
        }
        System.arraycopy(hkSupSdkProperties.getAlarmServerListenIp().getBytes(), 0, netEhomeAlarmListenParam.struAddress.szIP, 0, hkSupSdkProperties.getAlarmServerListenIp().length());
        if (Short.parseShort(hkSupSdkProperties.getAlarmServerType().toString()) == 2) {
            netEhomeAlarmListenParam.struAddress.wPort = Short.parseShort(hkSupSdkProperties.getAlarmServerListenTcpPort().toString());
            netEhomeAlarmListenParam.byProtocolType = 2; //协议类型：0- TCP，1- UDP, 2-MQTT
        } else {
            netEhomeAlarmListenParam.struAddress.wPort = Short.parseShort(hkSupSdkProperties.getAlarmServerListenUdpPort().toString());
            netEhomeAlarmListenParam.byProtocolType = 1; //协议类型：0- TCP，1- UDP, 2-MQTT
        }
        netEhomeAlarmListenParam.fnMsgCb = cbEHomeMsgCallBack;
        netEhomeAlarmListenParam.byUseCmsPort = 0; //是否复用CMS端口：0- 不复用，非0- 复用
        netEhomeAlarmListenParam.write();

        //启动报警服务器监听
        alarmHandle = hcEHomeAlarm.NET_EALARM_StartListen(netEhomeAlarmListenParam);
        if (alarmHandle < 0) {
            log.info("NET_EALARM_StartListen failed, error:" + hcEHomeAlarm.NET_EALARM_GetLastError());
            hcEHomeAlarm.NET_EALARM_Fini();
        } else {
            String AlarmListenInfo = new String(netEhomeAlarmListenParam.struAddress.szIP).trim() + "_" + netEhomeAlarmListenParam.struAddress.wPort;
            log.info("报警服务器：{}, NET_EALARM_StartListen succeed", AlarmListenInfo);
        }
    }

}
